#include <libwccl/values/matchvector.h>
#include <libpwrutils/foreach.h>
#include <sstream>

namespace Wccl {

const char* MatchVector::type_name = "MatchVector";

std::string MatchVector::to_raw_string() const
{
	std::stringstream ss;
	ss << "MATCH(";
	bool comma = false;
	foreach (const boost::shared_ptr<Match>& m, matches_) {
		if (comma) {
			ss << ",";
		}
		ss << m->to_raw_string();
		comma = true;
	}
	ss << ")";
	return ss.str();
}

std::string MatchVector::var_repr(const std::string &var_name)
{
	return Match::var_repr(var_name);
}

Position MatchVector::first_token() const
{
	if (matches_.empty()) {
		return Position(Position::Nowhere);
	} else {
		Position p = matches_.front()->first_token();
		for (size_t i = 1; i < matches_.end(); ++i) {
			Position c = matches_[i]->first_token();
			if (c.get_value() < p.get_value()) {
				p = c;
			}
		}
		return p;
	}
}

Position MatchVector::last_token() const
{
	if (matches_.empty()) {
		return Position(Position::Nowhere);
	} else {
		Position p = matches_.front()->last_token();
		for (size_t i = 1; i < matches_.end(); ++i) {
			Position c = matches_[i]->last_token();
			if (c.get_value() > p.get_value()) {
				p = c;
			}
		}
		return p;
	}
}

bool MatchVector::empty() const
{
	foreach (const boost::shared_ptr<Match>& m, matches_) {
		if (!m->empty()) {
			return false;
		}
	}
	return true;
}

void MatchVector::append(const boost::shared_ptr<Match> &m)
{
	matches_.push_back(m);
}


} /* end ns Wccl */
